# -*- coding:utf-8 -*-
import sys
import codecs

import jinja2

import publickey.yamldata as yd


template = jinja2.Template(u'''\
{% for id, x in items %}
Host {{ id }}
  {% if x.title %}# {{ x.title }}{% endif %}
  HostName {{ x.hostname }}
  Port {{ x.port|default(22) }}
  User {{ x.user|default('ubuntu') }}
  IdentityFile {{ x.identityfile|default('~/.ssh/id_rsa') }}
{% endfor %}
ServerAliveInterval 120
''')


def generate(config):
    doc = yd.load(config.filepath)
    items = sorted(
        yd.filter_by_tags(doc, config.tags),
        # ((name, dct), ...)
        key=lambda x: (yd.first_tag(x[1]), x[0]),
    )
    text = template.render(items=items)
    codecs.getwriter('utf-8')(sys.stdout).write(text)



# -*- coding:utf-8 -*-
import argparse

import api
import sshconfig


def make_config():
    parser = argparse.ArgumentParser()
    sub = parser.add_subparsers()

    getter = sub.add_parser('echo')
    getter.add_argument('filepath')
    getter.set_defaults(func=api.echo)

    getter = sub.add_parser('get')
    getter.add_argument('filepath')
    getter.add_argument('host')
    getter.set_defaults(func=api.download_from_github)

    putter = sub.add_parser('put')
    putter.add_argument('hosts', nargs='*')
    putter.add_argument('-t', '--tag', dest='tag')
    putter.add_argument('-s', '--src-file', dest='keyfile')
    putter.add_argument('-e', '--env-file', dest='filepath')
    putter.set_defaults(func=api.put_authorized_keys)

    setup = sub.add_parser('config')
    setup.add_argument('filepath')
    setup.add_argument('-t', '--tag', nargs='+', dest='tags', default=None)
    setup.set_defaults(func=sshconfig.generate)

    return parser.parse_args()


def main():
    config = make_config()
    config.func(config)


if __name__ == '__main__':
    main()



# -*- coding:utf-8 -*-
import yaml

empty = (None,)


def tags(entry):
    lst = entry.get('tags')
    if isinstance(lst, basestring):
        lst = [lst]
    return lst


def first_tag(entry):
    lst = tags(entry) or empty
    return lst[0]


def load(filepath):
    with open(filepath, 'r') as fp:
        return yaml.safe_load(fp.read())


def _all(doc):
    return ((name, x) for name, x in doc.items() if 'hostname' in x)


def filter_by_tags(doc, tags):
    items = _all(doc)
    if tags:
        criterion = set(tags)

        def filter_by_tags(iterable):
            for name, x in iterable:
                _tags = x.get('tags')
                if isinstance(_tags, basestring):
                    _tags = [_tags]
                if criterion <= set(_tags):
                    yield name, x
        items = list(filter_by_tags(items))
    return items



# -*- coding:utf-8 -*-
import sys
import codecs
from os.path import expanduser
from contextlib import closing
from tempfile import NamedTemporaryFile

import paramiko
import yaml
import requests

import publickey.yamldata as yd


def connect_params(config, host):
    _, params = config.lookup(host), {}

    def value_if(x, y, func=None):
        if x in _:
            params[y] = _[x] if not func else func(_[x])

    value_if('hostname', 'hostname')
    value_if('user', 'username')
    value_if('port', 'port', int)
    return params


def echo(config):
    filepath = config.filepath
    assert filepath
    doc = yd.load(filepath)
    stdout = sys.stdout

    with open(expanduser('~/.ssh/config'), 'r') as fp:
        config = paramiko.SSHConfig()
        config.parse(fp)

    with closing(paramiko.SSHClient()) as client:
        client.load_host_keys(expanduser('~/.ssh/known_hosts'))
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        items = [(yd.first_tag(x), name, x) for name, x in yd._all(doc)]
        for envname, name, dct in sorted(items, key=lambda x: x[0]):
            params = connect_params(config, name)
            stdout.write('%s: %s/%s ==> ' % (envname, name, dct['hostname']))
            try:
                client.connect(**params)
            except:
                stdout.write('Failed\n')
            else:
                stdout.write('OK\n')


def put_authorized_keys(config):
    stdout = codecs.getwriter('utf-8')(sys.stdout)
    keyfile = config.keyfile
    filepath = config.filepath
    tag = config.tag
    hosts = config.hosts

    assert bool(keyfile) ^ bool(filepath)
    assert bool(tag) ^ bool(hosts)
    if tag:
        assert filepath
        doc = yd.load(filepath)
        hosts = [name for name, _ in yd.filter_by_tags(doc, [tag])]

    with open(expanduser('~/.ssh/config'), 'r') as fp:
        config = paramiko.SSHConfig()
        config.parse(fp)

    stdout.write('***** Start deploying authorized_keys *****\n')
    with closing(paramiko.SSHClient()) as client:
        client.load_host_keys(expanduser('~/.ssh/known_hosts'))
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        for host in hosts:
            params = connect_params(config, host)
            if filepath:
                temp = NamedTemporaryFile()
                stdout.write(
                    '    ==> downloading \'%s/%s\' members publickeys '
                    'from github.com.\n' %
                    (host, params['hostname'])
                )
                _download_from_github(filepath, host, dest=temp)
                temp.seek(0)
                authorized_keys = temp.name
            else:
                authorized_keys = keyfile

            client.connect(**params)
            with closing(client.open_sftp()) as sftp:
                stdout.write(
                    '    ==> uploading authorized_keys to \'%s/%s\'.\n' %
                    (host, params['hostname'])
                )
                sftp.put(authorized_keys, '.ssh/authorized_keys')
    stdout.write('***** Finished *****\n')


def download_from_github(config, dest=None):
    return _download_from_github(config.filepath, config.host, dest)


def _download_from_github(filepath, host, dest):
    if not dest:
        dest = sys.stdout

    with open(filepath, 'r') as fp:
        doc = yaml.safe_load(fp.read())

    def _members(host):
        data = doc[host]
        assert 'members' in data
        return data['members']

    members = _members(host)
    for keystring in iterkeystrings(members):
        dest.write(keystring)


def iterkeystrings(members):
    for member in members:
        url = 'https://github.com/{0}.keys'.format(member)
        with closing(requests.get(url)) as response:
            assert response.ok
            items = enumerate(response.text.split('\n'), start=1)
            for number, line in items:
                yield '{0} {1}@github.com.{2}\n'.format(
                    line, member, number
                )



# -*- coding:utf-8 -*-
import os
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()


requirements = [
    'requests',
    'paramiko',
    'pyyaml',
    'flake8',
    'jinja2',
]

tests_require = [
    ]

entry_points = {
    'console_scripts': [
        'publickey=publickey:main',
    ],
}


setup(
    name='publickey',
    version='0.2.2',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    description='Control downloading/uploading publickeys.',
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'License :: OSI Approved :: MIT License',
    ],
    keywords='',
    author='imagawa_yakata(oyakata)',
    author_email='imagawa.hougikumaru@gmail.com',
    url='https://bitbucket.org/imagawa_yakata/publickey',
    license='MIT',
    tests_require=tests_require,
    install_requires=requirements,
    entry_points=entry_points,
)
